<?php
/***************************************************************
*  Copyright notice
*  
*  (c) 2004 Alexander Langer <alex@big.endian.de>
*  (c) 2003 Juraj Benadik (gastan@gastan.sk)
*  (c) 2002 Mads Jensen (mads.jensen@afs.org)
*  All rights reserved
*
*  This script is part of the Typo3 project. The Typo3 project is 
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/** 
 * Plugin 'Extended Calendar' for the 'calendar' extension.
 *
 * @author	Alexander Langer <alex@big.endian.de>
 * @author	Juraj Benadik (GASTAN)
 * @author	Mads-Peter Brunn Jensen
 */

require_once(PATH_tslib."class.tslib_pibase.php");

/* This corrensponds to the values you can set in the backend */
define("CALENDAR_VIEW_MONTH", 1);
define("CALENDAR_VIEW_WEEK", 2);
define("CALENDAR_VIEW_DAY", 3);
define("CALENDAR_VIEW_UPCOMING", 5);
define("CALENDAR_VIEW_EVENT", 6);

define("SECONDS_PER_HOUR", 3600);
define("SECONDS_PER_DAY", SECONDS_PER_HOUR * 24);
define("SECONDS_PER_WEEK", SECONDS_PER_DAY * 7);


class tx_calendar_pi1 extends tslib_pibase {
	var $prefixId = "tx_calendar_pi1";		// Same as class name
	var $scriptRelPath = "pi1/class.tx_calendar_pi1.php";	// Path to this script relative to the extension dir.
	var $extKey = "calendar";	// The extension key.

	var $dbTable = "tx_calendar_item";
	var $allowCaching = 1;

	var $EventsForSummary = array();
	
	function initdefaults($content, $conf) {
		$this->conf=$conf;
		$this->pi_setPiVarDefaults();
		$this->pi_loadLL();
		$this->internal["currentTable"] = $this->dbTable;

		// Do we have a current view timestamp already? if not, use todays
		if (!$this -> getTime()) {
			$this->setTime($this->getTodayTime());
		}

		if ($this->piVars['showUid'] && $this->getCurrentView() != CALENDAR_VIEW_EVENT) {
			unset($this->piVars['showUid']);
		}

		// same for view, use default view as given in Backend
		if (!$this->getCurrentView()) {
			$this->setCurrentView($this->cObj->data["tx_calendar_whattoshow"]);
		}
		$pid_list = $this->generatePidlistRecursive($this->cObj->data["pages"], $this->cObj->data["recursive"]);
		$this->setPidlist($pid_list);

		$this->loadCategories();
		$this->loadOrganizers();
	}

	function main($content,$conf)	{
		$this->initdefaults($content, $conf);

		$content .= $this -> pi_getEditPanel();

		switch ($this->getCurrentView()) {
			case CALENDAR_VIEW_WEEK:
				$content.= $this->displayWeek();
				break;
			case CALENDAR_VIEW_DAY:
				$content.= $this->displayDay();
				break;
			case CALENDAR_VIEW_EVENT:
				$content .= $this->displayEvent($this->pi_getRecord($this->dbTable, intval($this->piVars['showUid'])));
				break;
			case CALENDAR_VIEW_UPCOMING:
				$content.= $this->displayUpcoming($content, $conf);
				break;
			case CALENDAR_VIEW_MONTH:
			default :
				$content.= $this->displayMonth();
				break;
		}
		return $this->pi_wrapInBaseClass($content);
	}


	/**
	 * Loading the organizers into an internal array, $this->organizers
	 */
	function loadOrganizers() {
		$query = $this->pi_organizersUsed("tx_calendar_organizer");
		$res = mysql(TYPO3_db, $query);
		if (mysql_errno()) {
			die(mysql_error());
		}

		$this->organizers = array();

		while ($row = mysql_fetch_assoc($res)) {
			$this->organizers[$row["uid"]] = $row;
		}
	}

	/**
	 * Loading the categories into an internal array, $this->categories
	 */
	function loadCategories() {
		$query = $this->pi_categoriesUsed("tx_calendar_cat");
		$res = mysql(TYPO3_db, $query);
		if (mysql_errno()) {
			die(mysql_error());
		}

		$this->categories = array();

		while ($row = mysql_fetch_assoc($res)) {
			$this->categories[$row["uid"]] = $row;
		}
	}

	/**
	 * (Maybe bad name for this function) construct SQL-query for
	 * organizers
	 */
	function pi_organizersUsed($cat_table) {
		// XXX should be modified to return only organizers that are really in use.  
		$pidList = $this->getPidlist();
		$query = "SELECT ".$cat_table.".* "."\n"."FROM $cat_table WHERE $cat_table.pid IN ($pidList)".$this->cObj->enableFields($cat_table)."\n";
		return $query;
	}
	/**
	 * (Maybe bad name for this function) construct SQL-query for
	 * categories.
	 */
	function pi_categoriesUsed($cat_table) {
		// XXX should be modified to return only categories that are really in use.  
		$pidList = $this->getPidlist();
		$query = "SELECT ".$cat_table.".* "."\n"."FROM $cat_table WHERE $cat_table.pid IN ($pidList)".$this->cObj->enableFields($cat_table)."\n";
		return $query;
	}

	//function substituteMarkerArrayCallbackI


	/**
	 * get common wrappedSubpartArrays, such as LINK_CURRENT_MONTH
	 */
	function getCommonMarkerArray($lConf, $timestamp, $theMarkers = array()) {
		$markerArray = array();
		$subpartArray = array();
		$wrappedSubpartArray = array();

		foreach($theMarkers as $marker) {
			switch($marker) {

			/* Switch the view, but keep the date */
			case "###LINK_SWITCH_TO_MONTH###":
				$wrappedSubpartArray["###LINK_SWITCH_TO_MONTH###"] = $this->createLinkWrap(
					$this->pi_linkTP_keepPIvars_url(array("view" => CALENDAR_VIEW_MONTH), $this->allowCaching));
				break;
			case "###LINK_SWITCH_TO_WEEK###":
				$wrappedSubpartArray["###LINK_SWITCH_TO_WEEK###"] = $this->createLinkWrap( 
					$this->pi_linkTP_keepPIvars_url(array("view" => CALENDAR_VIEW_WEEK), $this->allowCaching));
				break;
			case "###LINK_SWITCH_TO_DAY###":
				$wrappedSubpartArray["###LINK_SWITCH_TO_DAY###"] = $this->createLinkWrap( 
					$this->pi_linkTP_keepPIvars_url(array("view" => CALENDAR_VIEW_DAY), $this->allowCaching));
				break;
			case "###LINK_SWITCH_TO_OVERVIEW###":
				$wrappedSubpartArray["###LINK_SWITCH_TO_OVERVIEW###"] = $this->createLinkWrap( 
					$this->pi_linkTP_keepPIvars_url(array("view" => CALENDAR_VIEW_QUICK), $this->allowCaching));
				break;

			/* Switch the date, but keep the view */
			case "###LINK_TODAY###":
				$newvars = array("viewtime" => $this->getTodayTime());
				$wrappedSubpartArray["###LINK_TODAY###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_MONTH###":
				$newvars = array("viewtime" => $timestamp, "view" => CALENDAR_VIEW_MONTH);
				$wrappedSubpartArray["###LINK_MONTH###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_DAY###":
				$newvars = array("viewtime" => $timestamp, "view" => CALENDAR_VIEW_DAY);
				$wrappedSubpartArray["###LINK_DAY###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_WEEK###":
				$newvars = array("viewtime" => $timestamp, "view" => CALENDAR_VIEW_WEEK);
				$wrappedSubpartArray["###LINK_WEEK###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###CURRENT_WEEK###":
				$markerArray["###CURRENT_WEEK###"] = $this->formatStr($this->cObj->stdWrap($timestamp, $lConf['weekTitleWrap.']));
				break;
			case "###CURRENT_WEEK_START###":
				$markerArray["###CURRENT_WEEK_START###"] = $this->formatStr($this->cObj->stdWrap($timestamp, $lConf['weekTitleStartDateWrap.']));
				break;
			case "###CURRENT_WEEK_END###":
				$markerArray["###CURRENT_WEEK_END###"] = $this->formatStr($this->cObj->stdWrap($timestamp + 604800 - 1, $lConf['weekTitleEndDateWrap.']));
				break;

			case "###CURRENT_MONTH###":
				$tmstmp = mktime(0, 0, 0, $this->getCurrentMonth($timestamp), 1, $this->getCurrentYear($timestamp));
				$markerArray["###CURRENT_MONTH###"]= $this->formatStr($this->cObj->stdWrap($tmstmp, $lConf['currentMonthWrap.']));
				break;
			case "###PREV_MONTH###":
				$markerArray["###PREV_MONTH###"]= $this->formatStr($this->cObj->stdWrap($this->getPrevMonthTime($timestamp), $lConf['prevMonthWrap.']));
				break;
			case "###NEXT_MONTH###":
				$markerArray["###NEXT_MONTH###"]= $this->formatStr($this->cObj->stdWrap($this->getNextMonthTime($timestamp), $lConf['nextMonthWrap.']));
				break;
			case "###PREV_YEAR###":
				$markerArray["###PREV_YEAR###"]= $this->formatStr($this->cObj->stdWrap($this->getPrevYearTime($timestamp), $lConf['prevYearWrap.']));
				break;
			case "###NEXT_YEAR###":
				$markerArray["###NEXT_YEAR###"]= $this->formatStr($this->cObj->stdWrap($this->getNextYearTime($timestamp), $lConf['nextYearWrap.']));
				break;
			case "###PREV_DAY###":
				$markerArray["###PREV_DAY###"] = $this->formatStr($this->cObj->stdWrap($timestamp - 86400, $lConf['prevDayWrap.']));
				break;
			case "###NEXT_DAY###":
				$markerArray["###NEXT_DAY###"] = $this->formatStr($this->cObj->stdWrap($timestamp + 86400, $lConf['nextDayWrap.']));
				break;
			case "###PREV_WEEK###":
				$markerArray["###PREV_WEEK###"] = $this->formatStr($this->cObj->stdWrap($timestamp - 604800, $lConf['prevWeekWrap.']));
				break;
			case "###NEXT_WEEK###":
				$markerArray["###NEXT_WEEK###"] = $this->formatStr($this->cObj->stdWrap($timestamp + 604800, $lConf['nextWeekWrap.']));
				break;

			case "###LINK_PREV_DAY###":
				// Hmm, I'd really like to wrap the link into a typolinkWrap, but there is no way to keep the PIvars, so do it directly
				$newvars = array("viewtime" => $this->getTime() - 86400);
				$wrappedSubpartArray["###LINK_PREV_DAY###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_NEXT_DAY###":
				$newvars = array("viewtime" => $this->getTime() + 86400);
				$wrappedSubpartArray["###LINK_NEXT_DAY###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_PREV_WEEK###":
				$newvars = array("viewtime" => $this->getTime() - 604800);
				$wrappedSubpartArray["###LINK_PREV_WEEK###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_NEXT_WEEK###":
				$newvars = array("viewtime" => $this->getTime() + 604800);
				$wrappedSubpartArray["###LINK_NEXT_WEEK###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_PREV_MONTH###":
				$newvars = array("viewtime" => $this->getPrevMonthTime($timestamp));
				$wrappedSubpartArray["###LINK_PREV_MONTH###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_NEXT_MONTH###":
				$newvars = array("viewtime" => $this->getNextMonthTime($timestamp));
				$wrappedSubpartArray["###LINK_NEXT_MONTH###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_PREV_YEAR###":
				$newvars = array("viewtime" => $this->getPrevYearTime($timestamp));
				$wrappedSubpartArray["###LINK_PREV_YEAR###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###LINK_NEXT_YEAR###":
				$newvars = array("viewtime" => $this->getNextYearTime($timestamp));
				$wrappedSubpartArray["###LINK_NEXT_YEAR###"]= $this->createLinkWrap($this->pi_linkTP_keepPIvars_url($newvars, $this->allowCaching));
				break;
			case "###CATEGORY_SELECT###":
				$displayCategorySelect = intval($lConf['displayCategorySelect']);
				$markerArray["###CATEGORY_SELECT###"] = $this ->makeCategorySelect($this->getCurrentCategory());
				break;
			default:
			}
		}

		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}

	/**
	 * requires $this->internal[currenRow] to be set
	 */
	function getMarkerArrayEventWise($lConf, $theMarkers) {
		$markerArray = array();
		$subpartArray = array();
		$wrappedSubpartArray = array();

		foreach($theMarkers as $marker) {
			switch($marker) {
			case "###EVENT_TITLE###":
				$markerArray["###EVENT_TITLE###"] = $this->pi_getEditIcon($this->formatStr($this->getFieldContent("title", $lConf)), "");
				break;
			case "###EVENT_TEASER###":
				$markerArray["###EVENT_TEASER###"] = $this->pi_getEditIcon($this->formatStr($this->getFieldContent("teaser", $lConf)), "");
				break;
			case "###EVENT_DESCRIPTION###":
				$markerArray["###EVENT_DESCRIPTION###"] = $this->formatStr($this->getFieldContent("descr", $lConf));
				break;

			case "###EVENT_STARTDATE###":
				$markerArray["###EVENT_STARTDATE###"] = $this->formatStr($this->getFieldContent("start_date", $lConf));
				break;
			case "###EVENT_STARTTIME###":
				$markerArray["###EVENT_STARTTIME###"] = $this->formatStr($this->getFieldContent("start_time", $lConf));
				break;
			case "###EVENT_ENDDATE###":
				$markerArray["###EVENT_ENDDATE###"] = $this->formatStr($this->getFieldContent("end_date", $lConf));
				break;
			case "###EVENT_ENDTIME###":
				$markerArray["###EVENT_ENDTIME###"] = $this->formatStr($this->getFieldContent("end_time", $lConf));
				break;

			case "###EVENT_PLACE###":
				$markerArray["###EVENT_PLACE###"] = $this->formatStr($this->getFieldContent("place", $lConf));
				break;
			case "###EVENT_ADDRESS###":
				$markerArray["###EVENT_ADDRESS###"] = $this->formatStr($this->getFieldContent("address", $lConf));
				break;

			case "###EVENT_ORGANIZER###":
				$markerArray["###EVENT_ORGANIZER###"] = $this->formatStr($this->getFieldContent("organizer", $lConf));
				break;
			case "###EVENT_ORGANIZER_EMAIL###":
				$markerArray["###EVENT_ORGANIZER_EMAIL###"] = $this->formatStr($this->getFieldContent("organizer_email", $lConf));
				break;
			case "###EVENT_ORGANIZER_URL###":
				$markerArray["###EVENT_ORGANIZER_URL###"] = $this->formatStr($this ->getFieldContent("organizer_url", $lConf));
				break;
			case "###EVENT_ORGANIZER_IMAGE###":
				$markerArray["###EVENT_ORGANIZER_IMAGE###"] = $this->formatStr($this ->getFieldContent("organizer_image", $lConf));
				break;
			case "###EVENT_MOREINFO###":
				$markerArray["###EVENT_MOREINFO###"] = $this->formatStr($this ->getFieldContent("moreinfo", $lConf));
				break;
			case "###EVENT_URL###":
				$markerArray["###EVENT_URL###"] = $this->formatStr($this ->getFieldContent("url", $lConf));
				break;
			case "###EVENT_IMAGE###":
				$markerArray["###EVENT_IMAGE###"] = $this->getFieldContent("image", $lConf);
				break;
			case "###EVENT_CATEGORY###":
				$markerArray["###EVENT_CATEGORY###"] = $this->formatStr($this ->getFieldContent("category", $lConf));
				break;
			case "###EVENT_CATEGORY_IMAGE###":
				$markerArray["###EVENT_CATEGORY_IMAGE###"] = $this->getFieldContent("category_image", $lConf);
				break;

			case "###LINK_EVENT###":
				$newarray = array("view" => CALENDAR_VIEW_EVENT, "showUid" => $this->internal['currentRow']['uid']);
				$displayUid = $lConf['pidDisplayEvent'];
				if (!$displayUid) 
					$displayUid = $this->conf['pidDisplayEvent'];
				if ($displayUid) {
					// ok, stupid pi_base doesn't have something which lets us keep our pivars, but link to another page
					// we must unset caching, unfortunately, otherwise it won't work
					$myvars = $this->makePiVars($newarray);
					$myvars['no_cache'] = 1;
					$link = $this->pi_getPageLink($displayUid, "", $myvars);
					$wrappedSubpartArray["###LINK_EVENT###"] = $this->createLinkWrap($link);
				} else {
					$wrappedSubpartArray["###LINK_EVENT###"] = $this->createLinkWrap(
						$this->pi_linkTP_keepPIvars_url($newarray, $this->allowCaching));
				}
				break;
			default:
			}
		}


		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}

	function getMarkerArrayEvent($lConf, $timestamp, $theMarkers = array()) {
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getCommonMarkerArray($lConf, $timestamp, $theMarkers);

		list ($m, $s, $w) = $this->getMarkerArrayEventWise($lConf, $theMarkers);

		$markerArray = array_merge($m, $markerArray);
		$subpartArray = array_merge($s, $subpartArray);
		$wrappedSubpartArray = array_merge($w, $wrappedSubpartArray);
		
		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}
	
	function displayEvent($row, $lConf = null, $parentTemplate = null, $timestamp = null)	{
		if ($lConf == null)
			$lConf = $this->conf;
		$lConf = $lConf["displayEvent."];
		if ($parentTemplate == null) {
			$eventTemplate = $this->loadTemplate("###TOP_TEMPLATE_EVENT###");
		} else {
			$eventTemplate = $this->loadTemplate("###TEMPLATE_EVENT###", $parentTemplate);
		}
		if ($timestamp == null)
			$timestamp = $this->getTime();

		$this->internal["currentRow"] = $row;
		$this->EventsForSummary[$row['uid']] = $row;

		// This sets the title of the page for use in indexed search results:
		if ($this->internal["currentRow"]["title"])
			$GLOBALS["TSFE"]->indexedDocTitle = $this->internal["currentRow"]["title"];
				
		$theMarkers = $this->getUsedMarkers($eventTemplate);
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getMarkerArrayEvent($lConf, $timestamp, $theMarkers);

		$this->getExtraDisplay($lConf, $timestamp, $markerArray);

		$content.= $this->cObj->substituteMarkerArrayCached($eventTemplate, $markerArray, $subpartArray, $wrappedSubpartArray);

		return $content;
	}

	function getMarkerArrayDay($lConf, $timestamp, $theMarkers = array()) {
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getCommonMarkerArray($lConf, $timestamp, $theMarkers);

		$markerArray["###DAY_HEADER###"] = $this->formatStr($this->cObj->stdWrap($timestamp, $lConf['dayTitleWrap.']));

		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}

	function getEventsStartingIn($from, $to, $addWhere = "") {
		$where = "((start_date + start_time >= $from AND start_date + start_time < $to AND start_time != 0) $addWhere)";
		$query = $this->cObj->getQuery($this->dbTable, $this->getSelectConf($where));
		$res = mysql(TYPO3_db, $query);

		$rows = array();
		while ($row = mysql_fetch_assoc($res))
			$rows[] = $row;
		return $rows;
	}

	function displayDay($lConf = null, $parentTemplate = null, $timestamp = null)	{
		if ($lConf == null)
			$lConf = $this->conf;
		$lConf = $lConf["displayDay."];
		if ($parentTemplate == null)
			$template = $this->loadTemplate("###TOP_TEMPLATE_DAY###");
		else
			$template = $this->loadTemplate("###TEMPLATE_DAY###", $parentTemplate);
		if ($timestamp == null)
			$timestamp = $this->getTime();
		$hourMatrixTemplate = $this->loadTemplate("###HOUR_MATRIX###", $template);
		$overlappingTemplate = $this->loadTemplate("###OVERLAPPING_EVENTS###", $template);
		$to_nextTemplate = $this->loadTemplate("###TO_NEXT_DAY_EVENTS###", $template);
		$from_prevTemplate = $this->loadTemplate("###FROM_PREV_DAY_EVENTS###", $template);
		$fulldayTemplate = $this->loadTemplate("###FULL_DAY_EVENTS###", $template);

		$theMarkers = $this->getUsedMarkers($template);
		list ($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getMarkerArrayDay($lConf, $timestamp, $theMarkers); 

		$daytable = "";
		$starttime = intval($lConf['displayStart']);
		if ($lConf['displayEnd'] == "")
		    	$endtime = 23 * 3600;
		else
			$endtime = intval($lConf['displayEnd']);
		$interval = intval($lConf['displayInterval']);
		$skipempty = intval($lConf['skipEmptyHours']);
		if ($lConf["skipHourTable"] == "")
			$skipHourTable = false;
		else
			$skipHourTable = intval($lConf['skipHourTable']);
		if ($lConf["adjustStartEnd"] == "")
			$adjust = false;
		else
			$adjust = intval($lConf['adjustStartEnd']);
		if (intval($interval) == 0)
			$interval = 3600;

		$todayptr = 0;
		$todaysevents = $this->getEventsOfDay($timestamp);
		$today_num = count($todaysevents);
		if ($today_num > 0)
			$has_events = true;
		else
			$has_events = false;

		$overlapping = "";		// Events that start earlier and end later (not today)
		$from_prev = "";		// Events that come from the prev day and end today
		$to_next = "";			// Events that start today and last at least until tomorrow
		$fullday = "";			// Events from today and only from today
		$hourTableEvents = array();	// Events that need to be displayed hourly

		while ($todayptr < $today_num) {
			$row = $todaysevents[$todayptr];
			// if this has a start_time set (1st line), and is ending today (2nd line)
			// and we either may adjust starttime + endtime or the events are in the limit
			if (!$skipHourTable &&
				$row['start_date'] == $timestamp && $row['start_time'] != 0 &&
				( $row['end_date'] == 0 || $row['end_date'] == $timestamp)  &&
				(($row['start_time'] >= $starttime && $row['end_time'] <= $endtime) || $adjust)) {
				$hourTableEvents[] = $row;
				$todayptr++;
				continue;
			}

			if ($row['start_date'] < $timestamp) {
				// started early
				if ($row['end_date'] > $timestamp) {
					$theConf = 'overlappingEvent.';
					if ($overlapping != "")
						$overlapping .= $this->cObj->stdWrap("", $lConf[$theConf]['eventDelimWrap.']);
					$addevent = $this->displayEvent($row, $lConf[$theConf], $overlappingTemplate);
					$overlapping .= $addevent;
				} else {
					$theConf = 'fromPrevDayEvent.';
					if ($from_prev != "")
						$from_prev .= $this->cObj->stdWrap("", $lConf[$theConf]['eventDelimWrap.']);
					$addevent = $this->displayEvent($row, $lConf[$theConf], $from_prevTemplate);
					$from_prev .= $addevent;
				}
			} else {
				// starts today
				if ($row['end_date'] > $timestamp) {
					$theConf = 'toNextDayEvent.';
					if ($to_next != "")
						$to_next .= $this->cObj->stdWrap("", $lConf[$theConf]['eventDelimWrap.']);
					$addevent = $this->displayEvent($row, $lConf[$theConf], $to_nextTemplate);
					$to_next .= $addevent;
				} else {
					$theConf = 'fullDayEvent.';
					if ($fullday != "")
						$fullday .= $this->cObj->stdWrap("", $lConf[$theConf]['eventDelimWrap.']);
					$addevent = $this->displayEvent($row, $lConf[$theConf], $fulldayTemplate);
					$fullday .= $addevent;
				}
			}
			$todayptr++;
		}

		$subpartArray["###OVERLAPPING_EVENTS###"] = $this->cObj->stdWrap($overlapping, $lConf['overlappingAllEventsWrap.']);
		$subpartArray["###FROM_PREV_DAY_EVENTS###"] = $this->cObj->stdWrap($from_prev, $lConf['fromPrevDayAllEventsWrap.']);
		$subpartArray["###FULL_DAY_EVENTS###"] = $this->cObj->stdWrap($fullday, $lConf['fullDayAllEventsWrap.']);
		$subpartArray["###TO_NEXT_DAY_EVENTS###"] = $this->cObj->stdWrap($to_next, $lConf['toNextDayAllEventsWrap.']);

		$todaysevents = $hourTableEvents;
		$today_num = count($todaysevents);

		$matrix = array();
		$howmany = ceil(SECONDS_PER_DAY / $interval);
		for ($x = 0; $x < $howmany; $x++) {
			$matrix[$x] = 0;
		}

		$maxcols = 1;
		if ($today_num) {
			// there are events left, that are not fullday
			// save how many events we have shown per line already
			$eventsArray = array();


			for ($i = 0; $i < $today_num; $i++) {
				$row = $todaysevents[$i];

				if ($adjust && $row['start_time'] < $starttime) {
					// We shall adjust starttime, so do it
					$starttime = floor($row['start_time'] / $interval) * $interval;
				}
				if ($adjust && $row['start_time'] > $endtime) {
					// We shall adjust starttime, so do it
					$endtime = (1 + ceil($row['start_time'] / $interval)) * $interval;
				}

				// what line to save this in
				$line = floor(intval($row['start_time']) / $interval);

				// If this doesn't have an end-date specified, set it to the start's day
				if (intval($row['end_date']) == 0)
					$row['end_date'] = intval($row['start_date']);

				// If this doesn't have an end-time, or end_date is not today, let it last to the display-end
				if (intval($row['end_time']) == 0 || $row['end_date'] != $timestamp)
					$row['end_time'] = 24 * 3600;

				// now calculate how many rows we need to span
				$row['lastsRows'] = ceil(intval($row['end_time']) / $interval - $line);

				// save in Matrix how many events shown per hour
				for ($x = $line; $x < $line + $row['lastsRows']; $x++) {
					$matrix[$x]++;
				}


				// save current event
				$eventsArray[$line][] = $row;
				$maxline = $line;
			}
		}

		// adjust start/endtime, if we have events that don't fit into the standard display
		for ($i = 0; $i < $maxline; $i++) {
			if ($adjust) {
				if ($i * $interval > $endtime && count($eventsArray[$i]) > 0) {
					$endtime = ($i + 5) * $interval;
				}
			}
			if ($matrix[$i] > $maxcols)
				$maxcols = $matrix[$i];
		}

		// now display each hour
		if (!$skipHourTable) {
			$howmany = ceil($endtime / $interval);
			$i = floor($starttime / $interval);
			for (; $i < $howmany; $i++) {
				if ($skipempty && count($eventsArray[$i]) == 0)
					continue;
				$line = "";
				$line .= $this->formatStr($this->cObj->stdWrap($timestamp + $i * $interval, $lConf['hourHeaderWrap.']));

				if (!is_array($eventsArray[$i]))
					$eventsArray[$i] = array();

				foreach ($eventsArray[$i] as $row) {
					$addevent = $this->displayEvent($row, $lConf, $hourMatrixTemplate);

					$temp = $this->cObj->stdWrap($addevent, $lConf['singleEventWrap.']);

					$rowspan = $row['lastsRows'];

					$temp = str_replace("###rowspan###", "$rowspan", $temp);

					$line .= $temp;
				}

				for ($x = $matrix[$i]; $x < $maxcols; $x++) 
					$line .= $this->cObj->stdWrap($this->formatStr(""), $lConf['emptyEventWrap.']);

				$daytable.= $this->cObj->stdWrap($line, $lConf[($i % 2) ? 'hourLineOddWrap.' : 'hourLineEvenWrap.']);
			}
			$subpartArray["###HOUR_MATRIX###"] = $this->cObj->stdWrap($daytable, $lConf['hourMatrixWrap.']);
		}

		$subpartArray["###TEMPLATE_EVENT###"] = "";
		$subpartArray["###TEMPLATE_FULLDAYEVENT###"] = "";

		$this->getExtraDisplay($lConf, $timestamp, $markerArray);

		$content = $this->cObj->substituteMarkerArrayCached($template, $markerArray, $subpartArray, $wrappedSubpartArray);

		if ($has_events) {
			$content = $this->cObj->stdWrap($content, $lConf['dayHasEventsWrap.']);
		} else {
			$content = $this->cObj->stdWrap($content, $lConf['dayNoEventsWrap.']);
		}

		if (date("Ynj", $timestamp) == date("Ynj"))
			$content = $this->cObj->stdWrap($content, $lConf['dayCurrentDayWrap.']);


		return $content;
	}

	function getEventsOfDay($timestamp) {
		$where = "(start_date = $timestamp OR (start_date < $timestamp AND end_date + end_time >= $timestamp))";

		//filter for category
		if ($this->getCurrentCategory() > 0) {
			$where .= ' AND (category = ' . $this->getCurrentCategory() . ') ';
		}
		$query = $this->cObj->getQuery($this->dbTable, $this->getSelectConf($where));
		$res = mysql(TYPO3_db, $query);
		$rows = array();
		while ($row = mysql_fetch_assoc($res))
			$rows[] = $row;
		return $rows;
	}

	function getMarkerArrayWeek($lConf, $timestamp, $theMarkers = array()) {

		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getCommonMarkerArray($lConf, $timestamp, $theMarkers);


		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}

	/**
	 * display the Week-View.  
	 */
	function displayWeek($lConf = null, $parentTemplate = null, $timestamp = null, $origTimeStamp = 0)	{
		if ($lConf == null)
			$lConf = $this->conf;
		$lConf = $lConf["displayWeek."];
		if ($parentTemplate == null)
			$template = $this->loadTemplate("###TOP_TEMPLATE_WEEK###");
		else
			$template = $this->loadTemplate("###TEMPLATE_WEEK###", $parentTemplate);
		if ($timestamp == null)
			$timestamp = $this->getTime();

		$dayTemplate = $this->loadTemplate("###DAY_MATRIX###", $template);

		if ($lConf['alignWeek'] == "")
			$alignWeek = true;
		else
			$alignWeek = $lConf['alignWeek'];

		if ($origTimeStamp == 0)
			$origTimeStamp = $timestamp;

		if ($alignWeek) {
			$dayOfWeek = $this->getDayOfTheWeek($timestamp);
			$timestamp = $timestamp - ($dayOfWeek-1) * SECONDS_PER_DAY;
		}

		$theMarkers = $this->getUsedMarkers($template);
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getMarkerArrayWeek($lConf, $timestamp, $theMarkers); 

		if ($alignWeek) {
			$dayOfWeek = $this->getDayOfTheWeek($timestamp);
			$timestamp = $timestamp - ($dayOfWeek-1) * SECONDS_PER_DAY;
		}

		$events = array();

		$weektable = "";
		for ($i = 0; $i < 7; $i++) { // First, fetch the events of this day
			$theDay = $this->displayDay($lConf, $dayTemplate, $timestamp + $i * SECONDS_PER_DAY);

			// XXX if you want to patch this, please add a TypoScript Var for the WeekEndDays, fetch the value from there and send me patches. thanks - Alex
			$dayOfWeek = $this->getDayOfTheWeek($timestamp + $i * SECONDS_PER_DAY);
			if ($dayOfWeek == 6 || $dayOfWeek == 7)
				$theDay = $this->cObj->stdWrap($theDay, $lConf['dayWeekEndWrap.']);

			if (date("n", $timestamp + $i * SECONDS_PER_DAY) != date("n", $origTimeStamp))
				$theDay = $this->cObj->stdWrap($theDay, $lConf['dayNotInMonthWrap.']);

			$theDay = $this->cObj->stdWrap($theDay, $lConf['dayAllWrap.']);

			$weektable .= $this->cObj->stdWrap($theDay, $lConf[($i % 2) ? 'dayOddWrap.' : 'dayEvenWrap.']);
		}

		$subpartArray["###DAY_MATRIX###"] = $this->cObj->stdWrap($weektable, $lConf['weekTableWrap.']);

		$subpartArray["###EVENT###"] = "";
		$subpartArray["###DAY_HEADER###"] = "";
		$subpartArray["###DAY_EVENTS###"] = "";

		$this->getExtraDisplay($lConf, $timestamp, $markerArray);

		$content = $this->cObj->substituteMarkerArrayCached($template, $markerArray, $subpartArray, $wrappedSubpartArray);

		return $content;
	}

	function getUsedMarkers($template) {
		$theMarkers = array();
		$temp = split("###", $template);
		for ($i = 1; $i < count($temp); $i += 2)
			$theMarkers[$temp[$i]] = "###" . $temp[$i] . "###";
		return $theMarkers;
	}


	function getMarkerArrayMonth($lConf, $timestamp, $theMarkers = array()) {
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getCommonMarkerArray($lConf, $timestamp, $theMarkers);
		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}

	function displayMonth($lConf = null, $parentTemplate = null, $timestamp = null)	{
		if ($lConf == null)
			$lConf = $this->conf;
		$lConf = $lConf["displayMonth."];
		if ($parentTemplate == null)
			$templatePrefix = "TOP_";
		$template = $this->loadTemplate("###".$templatePrefix."TEMPLATE_MONTH###", $parentTemplate);
		if ($timestamp == null)
			$timestamp = $this->getTime();

		$theMarkers = $this->getUsedMarkers($template);
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getMarkerArrayMonth($lConf, $timestamp, $theMarkers);

		// get the timestamps where the month begins and ends
		$monthBegin = $this->getMonthBeginTime($timestamp);
		$nextMonth = $this->getMonthBeginTime($this->getNextMonthTime($monthBegin));

		$mBWeekDay = $this->getDayOfTheWeek($monthBegin);
		$weekoff = (7 - $mBWeekDay + 1) * SECONDS_PER_DAY + SECONDS_PER_HOUR * 2; // DST! e.g. October 2004

		/* Create the Matrix - week-wise */
		$calstr = "";
		$calMatrixTemplate = $this->loadTemplate("###CALENDAR_MATRIX###", $template);
		$stop = false;
		$i = 0;
		while (1) {
			$theWeek = $monthBegin + $i * SECONDS_PER_WEEK;
		    	$week = $this->displayWeek($lConf, $calMatrixTemplate, $theWeek, $monthBegin);
			$calstr .= $this->cObj->stdWrap($week, $lConf['weekWrap.']);

			if ($theWeek + $weekoff >= $nextMonth)
				break;

			$i++;
		}

		$subpartArray["###CALENDAR_MATRIX###"] = $this->cObj->stdWrap($calstr, $lConf['calendarMatrixWrap.']);

		$subpartArray["###EVENT###"] = "";

		$this->getExtraDisplay($lConf, $timestamp, $markerArray);

		$content = $this->cObj->substituteMarkerArrayCached($template, $markerArray, $subpartArray, $wrappedSubpartArray);

		return $content;
	}

	function getExtraDisplay($lConf, $timestamp, &$markerArray) {
		$extra = $lConf['extraDisplay.'];
		$n = intval($extra['num']);
		for ($i = 1; $i <= $n; $i++) {
			$theContent = "";
			$sublConf = $extra["$i."];
			list($offday, $offweek, $offmonth, $offyear) = split(",", $sublConf['offset'], 4);;
			$offday = intval($offday);
			$offweek = intval($offweek);
			$offmonth = intval($offmonth);
			$offyear = intval($offyear);
			$thistimestamp = $timestamp;
			while($offday > 0) {
				$thistimestamp += SECONDS_PER_DAY;
				$offday--;
			}
			while($offday < 0) {
				$thistimestamp -= SECONDS_PER_DAY;
				$offday++;
			}
			while($offweek > 0) {
				$thistimestamp += SECONDS_PER_WEEK;
				$offweek--;
			}
			while($offweek < 0) {
				$thistimestamp -= SECONDS_PER_WEEK;
				$offweek++;
			}
			while($offmonth > 0) {
				$thistimestamp = $this->getNextMonthTime($thistimestamp);
				$offmonth--;
			}
			while($offmonth < 0) {
				$thistimestamp = $this->getPrevMonthTime($thistimestamp);
				$offmonth++;
			}
			// lacks code for year

			$function = $sublConf['function'];
			$theContent = $this->$function($sublConf, $this->loadTemplate('###' . $sublConf['subpart'] . '###'), $thistimestamp);
			$markerArray['###' . $sublConf['marker'] . '###'] = $theContent;
		}
	}

	function getMarkerArrayUpcoming($lConf, $timestamp, $theMarkers = array()) {
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getCommonMarkerArray($lConf, $timestamp, $theMarkers);
		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}
	
	function displayUpcoming($content, $conf) {
		$lConf = $this->conf["displayUpcoming."];

		$template = $this ->loadTemplate("###TOP_TEMPLATE_UPCOMING###");

		$theMarkers = $this->getUsedMarkers($template);
		list ($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getMarkerArrayUpcoming($lConf, $this->getTime(), $theMarkers); 
		$now = time();
		$where = "(
				-- events starting in the future
				start_date + start_time > $now
			OR
				-- events already started, but not yet finnished
				end_date + end_time > $now
			)";

		$query = $this->cObj->getQuery($this->dbTable, $this->getSelectConf($where));
		$res = mysql(TYPO3_db, $query);

		$max = $lConf['displayLimit'];
		$i = 0;
		$allEvents = "";
		while ($i < $max && $row = mysql_fetch_assoc($res)) {
			if ($i > 0)
				$allEvents .= $this->cObj->stdWrap("", $lConf['eventsDelimWrap.']);

			$addevent = $this->displayEvent($row, $lConf, $template);
			$allEvents .= $this->cObj->stdWrap($addevent, $lConf['singleEventWrap.']);
			$i++;
		}

		$markerArray["###ALLEVENTS###"] = $allEvents;
		$subpartArray["###TEMPLATE_EVENT###"] = "";

		$this->getExtraDisplay($lConf, $timestamp, $markerArray);

		$content.= $this->cObj->substituteMarkerArrayCached($template, $markerArray, $subpartArray, $wrappedSubpartArray);

		return $content;
	}

	function getMarkerArraySummary($lConf, $timestamp, $theMarkers = array()) {
		list($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getCommonMarkerArray($lConf, $timestamp, $theMarkers);
		return array($markerArray, $subpartArray, $wrappedSubpartArray);
	}
	
	function displaySummary($conf, $template, $timestamp) {
		$lConf = $conf["displaySummary."];

		$theMarkers = $this->getUsedMarkers($template);
		list ($markerArray, $subpartArray, $wrappedSubpartArray) = $this->getMarkerArraySummary($lConf, $timestamp, $theMarkers); 

		$content = "";
		$rows = $this->EventsForSummary;
		$this->EventsForSummary = array();

		$i = 0;
		if (!is_array($rows))
			$rows = array();
		foreach ($rows as $row) {
			if ($i > 0)
				$allEvents .= $this->cObj->stdWrap("", $lConf['eventsDelimWrap.']);
			$addevent = $this->displayEvent($row, $lConf, $template, $timestamp);
			$addevent = $this->cObj->stdWrap($addevent, $lConf['singleEventWrap.']);

			$enddate = $row['end_date'];
			if (!$enddate)
				$enddate = $row['start_date'];
			$endtime = $row['end_time'];
			if (!$endtime)
				$endtime = 24 * 3600;
			$endtime = $enddate + $endtime;
			if ($endtime < time())
				$expired = true;
			else
				$expired = false;

			// event in the past?
			$content .= $this->cObj->stdWrap($addevent, $lConf[$expired ? 'pastEventWrap.' : 'futureEventWrap.']);

			$i++;
		}

		return $content;
	}
	
	function getFieldContent($fN, $lConf)   {
		switch($fN) {
			case "uid":
				return $this->pi_list_linkSingle($this->internal["currentRow"][$fN],
					$this->internal["currentRow"]["uid"], $this->allowCaching);
				break;
			case "title":
				return $this->cObj->stdWrap($this->internal["currentRow"]["title"], $lConf['eventTitleWrap.']);
			case "descr":
				return $this->cObj->stdWrap($this->pi_RTEcssText($this->internal["currentRow"]["descr"]), $lConf['eventDescrWrap.']);
			case "start_date":
				return $this->cObj->stdWrap($this->internal["currentRow"]["start_date"] + $this->internal["currentRow"]["start_time"],
						$lConf["eventStartDateWrap."]);
			case "start_time":
				$value = $this->internal['currentRow']['start_time'];
				$value = ($value ? $value + $this->internal['currentRow']['start_date'] : '');
				return $this->cObj->stdWrap($value, $lConf["eventStartTimeWrap."]);
			case "end_date":
				$value = $this->internal['currentRow']['end_date'];
				$value = ($value ? $value : '');
				return $this->cObj->stdWrap($value, $lConf["eventEndDateWrap."]);
			case "end_time":
				$value = $this->internal['currentRow']['end_time'];
				if ($value) {
					$value = ($this->internal['currentRow']['end_date'] ?
						$value + $this->internal['currentRow']['end_date'] :
						$value + $this->internal['currentRow']['start_date']);
				} else {
					$value = '';
				}
				return $this->cObj->stdWrap($value, $lConf["eventEndTimeWrap."]);
			case "teaser":
				return $this->cObj->stdWrap(nl2br($this->internal["currentRow"]['teaser']), $lConf["eventTeaserWrap."]);

			case "organizer":
				if ($this->internal['currentRow']['organizer'])
					return $this->cObj->stdWrap($this->internal["currentRow"]['organizer'], $lConf["eventOrganizerWrap."]);
				if ($this->internal['currentRow']['organizer_mm']) {
					$oid = $this->internal['currentRow']['organizer_mm'];
					return $this->cObj->stdWrap($this->organizers[$oid]['title'], $lConf["eventOrganizerWrap."]);
				}
				return $this->cObj->stdWrap('', $lConf["eventOrganizerWrap."]);
			case "organizer_email":
				if ($this->internal['currentRow']['organizer_email'])
					return $this->cObj->stdWrap($this->internal["currentRow"]['organizer_email'], $lConf["eventOrganizerEmailWrap."]);
				if ($this->internal['currentRow']['organizer_mm']) {
					$oid = $this->internal['currentRow']['organizer_mm'];
					return $this->cObj->stdWrap($this->organizers[$oid]['email'], $lConf["eventOrganizerEmailWrap."]);
				}
				return $this->cObj->stdWrap("", $lConf["eventOrganizerEmailWrap."]);

			case "organizer_url":
				$theUrl = "";
				$theText = "";
				if ($this->internal['currentRow']['organizer_url'])
					$theUrl = $this->internal["currentRow"]['organizer_url'];
				else if ($this->internal['currentRow']['organizer_mm']) {
					$oid = $this->internal['currentRow']['organizer_mm'];
					$theUrl = $this->organizers[$oid]['url'];
				}
				if ($this->internal['currentRow']['organizer_url_text'])
					$theText = $this->internal["currentRow"]['organizer_url_text'];
				else if ($this->internal['currentRow']['organizer_mm']) {
					$oid = $this->internal['currentRow']['organizer_mm'];
					$theText = $this->organizers[$oid]['url_text'];
				}
				$url = $lConf['eventOrganizerUrl.'];
				$url['parameter'] = $theUrl;
				$text = $theText;
				if ($text) {
					$text = $this->formatStr($this->cObj->stdWrap($text, $lConf['eventOrganizerUrlTitleWrap.']));
				} else {
					$text = $this->cObj->typoLink_URL($url);
				}
				return $this->cObj->typolink($text, $url);

			case "organizer_image":
				$img = $lConf['eventOrganizerImage.'];
				$img['file'] = 'uploads/tx_calendar/' . $this->organizers[$this->internal['currentRow']['organizer_mm']]['image'];
				return $this->cObj->IMAGE($img);

			case "place":
				return $this->cObj->stdWrap($this->internal["currentRow"]['place'], $lConf["eventPlaceWrap."]);
			case "address":
				return $this->cObj->stdWrap(nl2br($this->internal["currentRow"]['address']), $lConf["eventAddressWrap."]);
			case "moreinfo":
				return $this->cObj->stdWrap($this->pi_RTEcssText($this->internal["currentRow"]['moreinfo']), $lConf["eventMoreInfoWrap."]);
			case "category":
				return $this->cObj->stdWrap($this->categories[$this->internal['currentRow']['category']]['title'], $lConf["eventCategoryWrap."]);
			case "category_image":
				$img = $lConf['eventCategoryImage.'];
				$img['file'] = 'uploads/tx_calendar/' . $this->categories[$this->internal['currentRow']['category']]['image'];
				return $this->cObj->IMAGE($img);

			case "url":
				$url = $lConf['eventUrl.'];
				$url['parameter'] = $this->internal['currentRow']['url'];
				$text = $this->internal['currentRow']['url_text'];
				if ($text) {
					$text = $this->formatStr($this->cObj->stdWrap($text, $lConf['eventUrlTitleWrap.']));
				} else {
					$text = $this->cObj->typoLink_URL($url);
				}
				return $this->cObj->typolink($text, $url);
			case "image":
				$img = $lConf['eventImage.'];
				$img['file'] = 'uploads/tx_calendar/' . $this->internal['currentRow']['image'];
				return $this->cObj->IMAGE($img);

                
            default:
                return $this->internal["currentRow"][$fN];
                break;
            }
        }


	


	function loadTemplate($theSubpart, $from = null) {
		if ($this->templateCode == null)
				$this->templateCode = $this->cObj->fileResource($this->conf["templateFile"]);
		if ($this->templateCode == "") {
			return '<h3>' . $this->extKey . ': no template file found:'.$this -> conf["templateFile"] . '</h3>';
		}
		if ($from == null) {
		    	return $this->cObj->getSubpart($this->templateCode, $theSubpart);
		} else {
		    	return $this->cObj->getSubpart($from, $theSubpart);
		}
	}


	function getSelectConf($where) {
		//get events
		$selectConf = Array();
		$selectConf["pidInList"] = $this -> pid_list;
		$selectConf["where"] = $where;
		$selectConf["orderBy"] = "start_date, start_time";
		return $selectConf;
	}


	/**
	 * Make the selectorbox with categories.
	*/
	function makeCategorySelect($selCat = "") {
		$this->loadCategories();
		$opt = array();

		$value = htmlentities($this->pi_linkTP_keepPIvars_url(array("category" => "")));

		/* Suck in label for "allCategories" from typoscript */
		if ($this->conf["allCategoriesMarker"])
			$opt[] = '<option value="'.$value.'">'.$this -> conf["allCategoriesMarker"].'</option>';
		else
			// if unset, use the one from locallang
			$opt[] = '<option value="'.$value.'">'.$this -> pi_getLL('allCategoriesMarker').'</option>';

		foreach ($this->categories as $cat) {
			$opt[] = '<option value="'.
				htmlentities($this->pi_linkTP_keepPIvars_url(array("category" => $cat['uid']), $this->allowCaching))
				.'"'.
				($selCat == $cat['uid'] ? " SELECTED" : "")
				.'>'.
				htmlentities($cat['title'])
				.'</option>';
		}
		return '<select onChange="document.location=this.options[this.selectedIndex].value;">'.implode("", $opt).'</select>';
	}



	/* -------------- small helper functions ----------------- */

	/**
	 * before displaying content, wrap everthing through general_stdWrap
	 */
	function formatStr($str)        {
		if (is_array($this->conf["general_stdWrap."]))  {
			$str = $this->cObj->stdWrap($str,$this->conf["general_stdWrap."]);
		}
		return $str;
	}

	/**
	 * put the given link in an array of anchor tags, for use with subpartArrays
	 */
	function createLinkWrap($theLink) {
		return array('<a href="' . $theLink . '">', '</a>');
	}


	/**
	 * Return seconds from 00:00 of a given date. XXX unused
	 * @param integer Unix timestamp
	 */
	function timestamp($res) {
		return 3600 * strftime("%H", $res) + 60 * strftime("%M", $res);
	}

	function setCurrentCategory($theValue) {
		$this->piVars["category"] = intval($theValue);
	}
	function getCurrentCategory() {
		return intval($this->piVars["category"]);
	}

	function setTime($theValue) {
		$this->piVars["viewtime"] = intval($theValue);
	}
	function getTime() {
		return intval($this->piVars["viewtime"]);
	}

	function getTodayTime() {
		return mktime(0, 0, 0, date('n'), date('j'), date('Y'));
	}


	function getCurrentDay($theTime) {
		return date('d', $theTime);
	}
	function getCurrentMonth($theTime) {
		return date('n', $theTime);
	}
	function getCurrentYear($theTime) {
		return date('Y', $theTime);
	}
	function getCurrentWeek($theTime) {
		return date('W', $theTime);
	}

	function getPrevMonthTime($theTime) {
		return mktime(0,0,0, $this->getPrevMonthMonth($theTime), $this->getCurrentDay($theTime), $this->getPrevMonthYear($theTime));
	}
	function getNextMonthTime($theTime) {
		return mktime(0,0,0, $this->getNextMonthMonth($theTime), $this->getCurrentDay($theTime), $this->getNextMonthYear($theTime));
	}
	function getPrevYearTime($theTime) {
		return mktime(0,0,0, $this->getCurrentMonth($theTime), $this->getCurrentDay($theTime), $this->getPrevYear($theTime));
	}
	function getNextYearTime($theTime) {
		return mktime(0,0,0, $this->getCurrentMonth($theTime), $this->getCurrentDay($theTime), $this->getNextYear($theTime));
	}

	/**
	 * return previous month
	 */
	function getPrevMonthMonth($theTime) {
		$month = $this->getCurrentMonth($theTime);
		return $month > 1 ? $month -1 : 12;
	}
	/**
	 * return previous month's year
	 */
	function getPrevMonthYear($theTime) {
		$year = $this->getCurrentYear($theTime);
		return $this->getCurrentMonth($theTime) > 1 ? $year : $year -1;
	}
	/**
	 * return next month
	 */
	function getNextMonthMonth($theTime) {
		$month = $this->getCurrentMonth($theTime);
		return $month < 12 ? $month +1 : 1;
	}
	/**
	 * return next month's year
	 */
	function getNextMonthYear($theTime) {
		$year = $this->getCurrentYear($theTime);
		return $this->getCurrentMonth($theTime) < 12 ? $year : $year +1;
	}
	/**
	 * return previous year
	 */
	function getPrevYear($theTime) {
		return $this->getCurrentYear($theTime) - 1;
	}
	/**
	 * return next year
	 */
	function getNextYear($theTime) {
		return $this->getCurrentYear($theTime) + 1;
	}
	
	/**
	 * get last day in month
	 */
	function getLastDayInMonth($theTime) {
		return date('t', $theTime);
	}
	
	/**
	 * return timestamp of first second in this month
	 */
	function getMonthBeginTime($theTime) {
		return mktime(0, 0, 0, $this->getCurrentMonth($theTime), 1, $this->getCurrentYear($theTime));
	}
	/**
	 * return timestamp of last second in this month
	 */
	function getMonthEndTime($theTime) {
		return mktime(23, 59, 59, $this->getCurrentMonth($theTime), $this->getLastDayInMonth($theTime), $this->getCurrentYear($theTime));
	}

	function getDayOfTheWeek($theTime) {
		$day =  date("w", $theTime);
		return ($day == 0 ? 7 : $day);
	}
	function getWeekOfTheYear($theTime) {
		return date("W", $theTime);
	}
	function getFirstDayOfTheWeek($theTime) {
		
	}


	function setCurrentView($theValue) {
		$this->piVars["view"] = intval($theValue);
	}
	function getCurrentView() {
		return intval($this->piVars["view"]);
	}

	/**
	 * Sets the pid_list internal var
	 */
	function setPidlist($pid_list) {
		$this->pid_list = $pid_list;
	}
	function getPidlist() {
		return $this->pid_list;
	}

	/**
	 * Extends the internal pid_list by the levels given by $recursive
	 */
	// Stolen from tt_news ;-)
	function generatePidlistRecursive($pid_list, $recursive)      {
		if ($pid_list == "")
			$pid_list = $GLOBALS["TSFE"]->id;
		if ($recursive) {               // get pid-list if recursivity is enabled
			$pid_list_arr = explode(",", $pid_list);
			$pid_list = "";
			while(list(, $val) = each($pid_list_arr))  {
				$pid_list .= $val . "," . $this->cObj->getTreeList($val, intval($recursive));
			}
			$pid_list = ereg_replace(",$", "", $pid_list);
		}
		return $pid_list;
	}


	function makePiVars($overwrite) {
		$params = array();

		foreach (array_keys($this->piVars) as $key) {
			$params[$this->prefixId . '[' .  $key . ']'] = $this->piVars[$key];
		}

		foreach (array_keys($overwrite) as $key) {
			$params[$this->prefixId . '[' .  $key . ']'] = $overwrite[$key];
		}

		return $params;
	}
}



if (defined("TYPO3_MODE") && $TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/calendar/pi1/class.tx_calendar_pi1.php"])	{
	include_once($TYPO3_CONF_VARS[TYPO3_MODE]["XCLASS"]["ext/calendar/pi1/class.tx_calendar_pi1.php"]);
}

?>
